<?php

/**
 * @file
 * Contains \Drupal\url_restrictions\EventSubscriber\UrlRestrictions.
 */

namespace Drupal\url_restrictions\EventSubscriber;

use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;

/**
 * Event Subscriber for Url Restrictions.
 */
class UrlRestrictions implements EventSubscriberInterface {

   /**
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $request;

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The kill switch.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */

  protected $killSwitch;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   * The current user.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   * The request service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   * The config service.
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $killSwitch
   * The page cache kill switch service.  
   */
  public function __construct(AccountProxyInterface $currentUser,
                              ConfigFactoryInterface $config,
                              RequestStack $request,
                              KillSwitch $killSwitch) {
    $this->currentUser = $currentUser;
    $this->config = $config;
    $this->request = $request;
    $this->killSwitch = $killSwitch;
  }

  /**
   * Code that should be triggered on event specified 
   */
  public function onRespond(GetResponseEvent $event) {   
    $response = $event->getResponse();    
    $request_uri = $this->request->getCurrentRequest()->server->get('REQUEST_URI');
    $path = explode("/" , $request_uri);    
    $arr_entity_tpye = $this->config->get('url_restrictions.settings')->get('entity_tpye');
    $redirect_path = $this->config->get('url_restrictions.settings')->get('url');
    $pages = $this->config->get('url_restrictions.settings')->get('pages');     
    $pages = explode(PHP_EOL ,$pages);    
    $role = $this->currentUser->getRoles();    
    if(!in_array('administrator' , $role)) {
      if(preg_match('/node\/(\d+)\/edit/', $request_uri, $matches) || preg_match('/taxonomy\/term\/(\d+)\/edit/', $request_uri, $matches) || preg_match('/user\/(\d+)\/edit/', $request_uri, $matches)) {
        $this->killSwitch->trigger();
        $response = new RedirectResponse($redirect_path);
        $response->send();
      }
      if(isset($path[1]) && in_array($path[1] , $arr_entity_tpye , TRUE) && $request_uri != $redirect_path) { 
        if(!in_array($request_uri, $pages)) {
           $response = new TrustedRedirectResponse($redirect_path);
           $response->addCacheableDependency(CacheableMetadata::createFromRenderArray([])->addCacheTags(['rendered']));
           $event->setResponse($response);
        }
      }
    }    
  }
  

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {    
    $events[KernelEvents::REQUEST][] = ['onRespond'];
    return $events;
  }

}